"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const path_1 = require("path");
const fs_1 = require("fs");
const util_1 = require("./util");
function checkType(typeOrFunction, value, options = {}) {
    return checkTypeCore(typeOrFunction, value, options);
}
exports.checkType = checkType;
/** @internal */
function checkTypeCore(typeOrFunction, value, options = {}) {
    let d;
    let sourceFile;
    const tsConfigFilePath = options.tsConfigFilePath || './tsconfig.json';
    if (!project) {
        project = new ts_simple_ast_1.Project({
            tsConfigFilePath,
            addFilesFromTsConfig: true,
        });
    }
    const { callerFile, allCallerFiles } = util_1.getCallerFile();
    if (!callerFile) {
        return {
            pass: false,
            error: `Caller source file cannot be found, aborting`,
            allCallerFiles,
        };
    }
    const callerSourceFile = project.getSourceFile(callerFile);
    if (!callerSourceFile) {
        return {
            pass: false,
            error: `Caller source must belong to ${tsConfigFilePath} project but ${callerFile} does not`,
            callerFile,
            allCallerFiles,
        };
    }
    if (options.verifyProject) {
        d = project.getPreEmitDiagnostics();
        if (d.length) {
            return {
                pass: false,
                error: `Given TypeScript project cannot have compilation errors, fix them and try again. Errors: ${util_1.formatDiagnostics(d)}`,
                callerFile,
                allCallerFiles,
            };
        }
    }
    else if (!options.dontVerifyFile) {
        d = callerSourceFile.getPreEmitDiagnostics();
        if (d.length) {
            return {
                pass: false,
                error: `Caller TypeScript file cannot have compilation errors, fix them and try again. Errors: ${util_1.formatDiagnostics(d)}`,
                callerFile,
                allCallerFiles,
            };
        }
    }
    //TODO: verify it's contained in project
    const folderName = path_1.dirname(callerFile);
    const fileName = `${options.folder || ''}${util_1.unique(path_1.basename(callerFile, '.ts'))}.ts`; // TODO: callerFile could be tsx
    const filePath = path_1.join(folderName, fileName);
    let testCode;
    const escapedValue = options.dontEscape ? value : util_1.escapeValue(value, options);
    if (escapedValue === undefined) {
        return {
            pass: false,
            error: `Value is not JSON and option enforceJsonValues was used`,
            callerFile,
            filePath,
            allCallerFiles,
        };
    }
    if (typeof typeOrFunction === 'string') {
        testCode = `const ${util_1.unique('variable')}: ${typeOrFunction} = ${escapedValue}`;
    }
    else {
        testCode = typeOrFunction(escapedValue);
    }
    const code = `
  ${fs_1.readFileSync(callerFile).toString()}
  ${testCode}`;
    sourceFile = project.createSourceFile(filePath, code);
    d = sourceFile.getPreEmitDiagnostics();
    const r = {
        pass: d.length > 0 ? false : true,
        failErrors: util_1.formatDiagnostics(d),
        code,
        testCode,
        callerFile,
        filePath,
        allCallerFiles,
    };
    if (options.printResult || (!r.pass && options.printResultIfFail)) {
        console.log(JSON.stringify(r, null, 2));
    }
    return r;
}
exports.checkTypeCore = checkTypeCore;
let project;
